# frozen_string_literal: true

# ActiveRecord makes use of autoload to load some of its components as-needed.
# This doesn"t work well with threaded environments, and causes uninitialized
# constants. To avoid this, this file manually requires the following files that
# are not required using `require "active_record"`. Trying to automatically
# require every file in ActiveRecord is slow and inefficient. Many of the files
# fail to require and there are some we don"t want to require. Thus, this is a
# manual list of requires.

# To re-build this list of requires, run the following:
#   bundle exec ruby script/determine_autoloaded_active_record_files.rb

# For compatibility, we require active record first
require "active_record"

if (
  ActiveRecord.version.segments[0] == 7 &&
  ActiveRecord.version.segments[1] >= 0
)
  require "active_record/aggregations"
  require "active_record/association_relation"
  require "active_record/associations"
  require "active_record/associations/alias_tracker"
  require "active_record/associations/association"
  require "active_record/associations/association_scope"
  require "active_record/associations/belongs_to_association"
  require "active_record/associations/belongs_to_polymorphic_association"
  require "active_record/associations/builder/association"
  require "active_record/associations/builder/belongs_to"
  require "active_record/associations/builder/collection_association"
  require "active_record/associations/builder/has_and_belongs_to_many"
  require "active_record/associations/builder/has_many"
  require "active_record/associations/builder/has_one"
  require "active_record/associations/collection_association"
  require "active_record/associations/collection_proxy"
  require "active_record/associations/disable_joins_association_scope"
  require "active_record/associations/foreign_association"
  require "active_record/associations/has_many_association"
  require "active_record/associations/has_many_through_association"
  require "active_record/associations/has_one_association"
  require "active_record/associations/has_one_through_association"
  require "active_record/associations/join_dependency"
  require "active_record/associations/join_dependency/join_association"
  require "active_record/associations/join_dependency/join_base"
  require "active_record/associations/preloader"
  require "active_record/associations/preloader/association"
  require "active_record/associations/preloader/batch"
  require "active_record/associations/preloader/branch"
  require "active_record/associations/preloader/through_association"
  require "active_record/asynchronous_queries_tracker"
  require "active_record/attribute_assignment"
  require "active_record/attribute_methods/before_type_cast"
  require "active_record/attribute_methods/dirty"
  require "active_record/attribute_methods/primary_key"
  require "active_record/attribute_methods/query"
  require "active_record/attribute_methods/read"
  require "active_record/attribute_methods/serialization"
  require "active_record/attribute_methods/time_zone_conversion"
  require "active_record/attribute_methods/write"
  require "active_record/attributes"
  require "active_record/autosave_association"
  require "active_record/base"
  require "active_record/coders/json"
  require "active_record/disable_joins_association_relation"
  require "active_record/encryption/cipher/aes256_gcm"
  require "active_record/encryption/derived_secret_key_provider"
  require "active_record/encryption/deterministic_key_provider"
  require "active_record/encryption/encrypted_attribute_type"
  require "active_record/encryption/encrypted_fixtures"
  require "active_record/encryption/encrypting_only_encryptor"
  require "active_record/encryption/envelope_encryption_key_provider"
  require "active_record/encryption/extended_deterministic_queries"
  require "active_record/encryption/extended_deterministic_uniqueness_validator"
  require "active_record/encryption/key"
  require "active_record/encryption/message"
  require "active_record/encryption/null_encryptor"
  require "active_record/encryption/properties"
  require "active_record/encryption/read_only_null_encryptor"
  require "active_record/encryption/scheme"
  require "active_record/future_result"
  require "active_record/internal_metadata"
  require "active_record/legacy_yaml_adapter"
  require "active_record/middleware/database_selector"
  require "active_record/middleware/shard_selector"
  require "active_record/null_relation"
  require "active_record/query_logs"
  require "active_record/relation/predicate_builder"
  require "active_record/relation/record_fetch_warning"
  require "active_record/result"
  require "active_record/runtime_registry"
  require "active_record/statement_cache"
  require "active_record/table_metadata"
  require "active_record/tasks/database_tasks"
  require "active_record/tasks/mysql_database_tasks"
  require "active_record/tasks/postgresql_database_tasks"
  require "active_record/tasks/sqlite_database_tasks"
  require "arel/collectors/bind"
  require "arel/collectors/composite"
  require "arel/collectors/substitute_binds"

elsif (
  ActiveRecord.version.segments[0] == 6 &&
  ActiveRecord.version.segments[1] > 0
)
  require "active_record/aggregations"
  require "active_record/association_relation"
  require "active_record/associations"
  require "active_record/associations/alias_tracker"
  require "active_record/associations/association"
  require "active_record/associations/association_scope"
  require "active_record/associations/belongs_to_association"
  require "active_record/associations/belongs_to_polymorphic_association"
  require "active_record/associations/builder/association"
  require "active_record/associations/builder/belongs_to"
  require "active_record/associations/builder/collection_association"
  require "active_record/associations/builder/has_and_belongs_to_many"
  require "active_record/associations/builder/has_many"
  require "active_record/associations/builder/has_one"
  require "active_record/associations/collection_association"
  require "active_record/associations/collection_proxy"
  require "active_record/associations/foreign_association"
  require "active_record/associations/has_many_association"
  require "active_record/associations/has_many_through_association"
  require "active_record/associations/has_one_association"
  require "active_record/associations/has_one_through_association"
  require "active_record/associations/join_dependency"
  require "active_record/associations/join_dependency/join_association"
  require "active_record/associations/join_dependency/join_base"
  require "active_record/associations/preloader"
  require "active_record/associations/preloader/association"
  require "active_record/associations/preloader/through_association"
  require "active_record/attribute_assignment"
  require "active_record/attribute_methods/before_type_cast"
  require "active_record/attribute_methods/dirty"
  require "active_record/attribute_methods/primary_key"
  require "active_record/attribute_methods/query"
  require "active_record/attribute_methods/read"
  require "active_record/attribute_methods/serialization"
  require "active_record/attribute_methods/time_zone_conversion"
  require "active_record/attribute_methods/write"
  require "active_record/attributes"
  require "active_record/autosave_association"
  require "active_record/base"
  require "active_record/coders/json"
  require "active_record/internal_metadata"
  require "active_record/legacy_yaml_adapter"
  require "active_record/middleware/database_selector"
  require "active_record/null_relation"
  require "active_record/relation/predicate_builder"
  require "active_record/relation/record_fetch_warning"
  require "active_record/result"
  require "active_record/runtime_registry"
  require "active_record/statement_cache"
  require "active_record/table_metadata"
  require "active_record/tasks/database_tasks"
  require "active_record/tasks/mysql_database_tasks"
  require "active_record/tasks/postgresql_database_tasks"
  require "active_record/tasks/sqlite_database_tasks"
  require "arel/collectors/bind"
  require "arel/collectors/composite"
  require "arel/collectors/substitute_binds"
elsif (
  ActiveRecord.version.segments[0] == 6 &&
  ActiveRecord.version.segments[1] == 0
)
  require "active_record/advisory_lock_base"
  require "active_record/associations/alias_tracker"
  require "active_record/associations/association"
  require "active_record/associations/association_scope"
  require "active_record/associations/belongs_to_association"
  require "active_record/associations/belongs_to_polymorphic_association"
  require "active_record/associations/builder/belongs_to"
  require "active_record/associations/builder/collection_association"
  require "active_record/associations/builder/has_and_belongs_to_many"
  require "active_record/associations/builder/has_many"
  require "active_record/associations/builder/has_one"
  require "active_record/associations/collection_association"
  require "active_record/associations/foreign_association"
  require "active_record/associations/has_many_association"
  require "active_record/associations/has_many_through_association"
  require "active_record/associations/has_one_association"
  require "active_record/associations/has_one_through_association"
  require "active_record/associations/join_dependency"
  require "active_record/associations/join_dependency/join_association"
  require "active_record/associations/join_dependency/join_base"
  require "active_record/associations/preloader"
  require "active_record/associations/preloader/association"
  require "active_record/associations/preloader/through_association"
  require "active_record/coders/json"
  require "active_record/internal_metadata"
  require "active_record/legacy_yaml_adapter"
  require "active_record/middleware/database_selector"
  require "active_record/null_relation"
  require "active_record/relation/predicate_builder"
  require "active_record/relation/record_fetch_warning"
  require "active_record/result"
  require "active_record/runtime_registry"
  require "active_record/statement_cache"
  require "active_record/table_metadata"
  require "active_record/tasks/database_tasks"
  require "active_record/tasks/mysql_database_tasks"
  require "active_record/tasks/postgresql_database_tasks"
  require "active_record/tasks/sqlite_database_tasks"
elsif ActiveRecord.version.segments[0] == 5
  require "active_record/aggregations"
  require "active_record/association_relation"
  require "active_record/associations"
  require "active_record/associations/alias_tracker"
  require "active_record/associations/association"
  require "active_record/associations/association_scope"
  require "active_record/associations/belongs_to_association"
  require "active_record/associations/belongs_to_polymorphic_association"
  require "active_record/associations/builder/association"
  require "active_record/associations/builder/belongs_to"
  require "active_record/associations/builder/collection_association"
  require "active_record/associations/builder/has_and_belongs_to_many"
  require "active_record/associations/builder/has_many"
  require "active_record/associations/builder/has_one"
  require "active_record/associations/collection_association"
  require "active_record/associations/collection_proxy"
  require "active_record/associations/foreign_association"
  require "active_record/associations/has_many_association"
  require "active_record/associations/has_many_through_association"
  require "active_record/associations/has_one_association"
  require "active_record/associations/has_one_through_association"
  require "active_record/associations/join_dependency"
  require "active_record/associations/join_dependency/join_association"
  require "active_record/associations/join_dependency/join_base"
  require "active_record/associations/preloader"
  require "active_record/associations/preloader/association"
  require "active_record/associations/preloader/through_association"
  require "active_record/attribute_assignment"
  require "active_record/attribute_decorators"
  require "active_record/attribute_methods/before_type_cast"
  require "active_record/attribute_methods/dirty"
  require "active_record/attribute_methods/primary_key"
  require "active_record/attribute_methods/query"
  require "active_record/attribute_methods/read"
  require "active_record/attribute_methods/serialization"
  require "active_record/attribute_methods/time_zone_conversion"
  require "active_record/attribute_methods/write"
  require "active_record/attributes"
  require "active_record/autosave_association"
  require "active_record/base"
  require "active_record/coders/json"
  require "active_record/internal_metadata"
  require "active_record/legacy_yaml_adapter"
  require "active_record/null_relation"
  require "active_record/relation/predicate_builder"
  require "active_record/relation/record_fetch_warning"
  require "active_record/result"
  require "active_record/runtime_registry"
  require "active_record/statement_cache"
  require "active_record/table_metadata"
  require "active_record/tasks/database_tasks"
  require "active_record/tasks/mysql_database_tasks"
  require "active_record/tasks/postgresql_database_tasks"
  require "active_record/tasks/sqlite_database_tasks"
else
  raise(
    "ActiveRecord #{ActiveRecord.version} isn't supported. "\
    "Switch to 5.X or 6.X",
  )
end

# There are also issues with requiring ActiveSupport. This doesn"t require every
# ActiveSupport file though, only ones we"ve seen cause problems.
require "active_support"
require "active_support/multibyte/chars"
